@@ -131,7 +131,7 @@ void do_open_filesys(int argc, char **argv)
 	char	*data_filename = 0;
 
 	reset_getopt();
-	while ((c = getopt (argc, argv, "iwfecb:s:d:D")) != EOF) {
+	while ((c = getopt (argc, argv, "iwfecb:s:d:")) != EOF) {
 		switch (c) {
 		case 'i':
 			open_flags |= EXT2_FLAG_IMAGE_FILE;
@@ -151,9 +151,6 @@ void do_open_filesys(int argc, char **argv)
 		case 'd':
 			data_filename = optarg;
 			break;
-		case 'D':
-			open_flags |= EXT2_FLAG_DIRECT_IO;
-			break;
 		case 'b':
 			blocksize = parse_ulong(optarg, argv[0],
 						"block size", &err);
@@ -223,28 +220,8 @@ static void close_filesystem(NOARGS)
 
 void do_close_filesys(int argc, char **argv)
 {
-	int	c;
-
-	if (check_fs_open(argv[0]))
+	if (common_args_process(argc, argv, 1, 1, "close_filesys", "", 0))
 		return;
-
-	reset_getopt();
-	while ((c = getopt (argc, argv, "a")) != EOF) {
-		switch (c) {
-		case 'a':
-			current_fs->flags &= ~EXT2_FLAG_MASTER_SB_ONLY;
-			break;
-		default:
-			goto print_usage;
-		}
-	}
-
-	if (argc > optind) {
-	print_usage:
-		com_err(0, 0, "Usage: close_filesys [-a]");
-		return;
-	}
-
 	close_filesystem();
 }
 
@@ -314,6 +291,9 @@ void do_show_super_stats(int argc, char *argv[])
 	int	c, header_only = 0;
 	int	numdirs = 0, first, gdt_csum;
 
+	gdt_csum = EXT2_HAS_RO_COMPAT_FEATURE(current_fs->super,
+					      EXT4_FEATURE_RO_COMPAT_GDT_CSUM);
+
 	reset_getopt();
 	while ((c = getopt (argc, argv, "h")) != EOF) {
 		switch (c) {
@@ -341,8 +321,6 @@ void do_show_super_stats(int argc, char *argv[])
 		return;
 	}
 
-	gdt_csum = EXT2_HAS_RO_COMPAT_FEATURE(current_fs->super,
-					      EXT4_FEATURE_RO_COMPAT_GDT_CSUM);
 	gdp = &current_fs->group_desc[0];
 	for (i = 0; i < current_fs->group_desc_count; i++, gdp++) {
 		fprintf(out, " Group %2d: block bitmap at %u, "
@@ -554,136 +532,6 @@ static void dump_blocks(FILE *f, const char *prefix, ext2_ino_t inode)
 	fprintf(f,"\n");
 }
 
-static int int_log10(unsigned long long arg)
-{
-	int     l = 0;
-
-	arg = arg / 10;
-	while (arg) {
-		l++;
-		arg = arg / 10;
-	}
-	return l;
-}
-
-#define DUMP_LEAF_EXTENTS	0x01
-#define DUMP_NODE_EXTENTS	0x02
-#define DUMP_EXTENT_TABLE	0x04
-
-static void dump_extents(FILE *f, const char *prefix, ext2_ino_t ino,
-			 int flags, int logical_width, int physical_width)
-{
-	ext2_extent_handle_t	handle;
-	struct ext2fs_extent	extent;
-	struct ext2_extent_info info;
-	int			op = EXT2_EXTENT_ROOT;
-	unsigned int		printed = 0;
-	errcode_t 		errcode;
-
-	errcode = ext2fs_extent_open(current_fs, ino, &handle);
-	if (errcode)
-		return;
-
-	if (flags & DUMP_EXTENT_TABLE)
-		fprintf(f, "Level Entries %*s %*s Length Flags\n",
-			(logical_width*2)+3, "Logical",
-			(physical_width*2)+3, "Physical");
-	else
-		fprintf(f, "%sEXTENTS:\n%s", prefix, prefix);
-
-	while (1) {
-		errcode = ext2fs_extent_get(handle, op, &extent);
-
-		if (errcode)
-			break;
-
-		op = EXT2_EXTENT_NEXT;
-
-		if (extent.e_flags & EXT2_EXTENT_FLAGS_SECOND_VISIT)
-			continue;
-
-		if (extent.e_flags & EXT2_EXTENT_FLAGS_LEAF) {
-			if ((flags & DUMP_LEAF_EXTENTS) == 0)
-				continue;
-		} else {
-			if ((flags & DUMP_NODE_EXTENTS) == 0)
-				continue;
-		}
-
-		errcode = ext2fs_extent_get_info(handle, &info);
-		if (errcode)
-			continue;
-
-		if (!(extent.e_flags & EXT2_EXTENT_FLAGS_LEAF)) {
-			if (extent.e_flags & EXT2_EXTENT_FLAGS_SECOND_VISIT)
-				continue;
-
-			if (flags & DUMP_EXTENT_TABLE) {
-				fprintf(f, "%2d/%2d %3d/%3d %*llu - %*llu "
-					"%*llu%*s %6u\n",
-					info.curr_level, info.max_depth,
-					info.curr_entry, info.num_entries,
-					logical_width,
-					extent.e_lblk,
-					logical_width,
-					extent.e_lblk + (extent.e_len - 1),
-					physical_width,
-					extent.e_pblk,
-					physical_width+3, "", extent.e_len);
-				continue;
-			}
-
-			fprintf(f, "%s(ETB%d):%lld",
-				printed ? ", " : "", info.curr_level,
-				extent.e_pblk);
-			printed = 1;
-			continue;
-		}
-
-		if (flags & DUMP_EXTENT_TABLE) {
-			fprintf(f, "%2d/%2d %3d/%3d %*llu - %*llu "
-				"%*llu - %*llu %6u %s\n",
-				info.curr_level, info.max_depth,
-				info.curr_entry, info.num_entries,
-				logical_width,
-				extent.e_lblk,
-				logical_width,
-				extent.e_lblk + (extent.e_len - 1),
-				physical_width,
-				extent.e_pblk,
-				physical_width,
-				extent.e_pblk + (extent.e_len - 1),
-				extent.e_len,
-				extent.e_flags & EXT2_EXTENT_FLAGS_UNINIT ?
-					"Uninit" : "");
-			continue;
-		}
-
-		if (extent.e_len == 0)
-			continue;
-		else if (extent.e_len == 1)
-			fprintf(f,
-				"%s(%lld%s):%lld",
-				printed ? ", " : "",
-				extent.e_lblk,
-				extent.e_flags & EXT2_EXTENT_FLAGS_UNINIT ?
-				"[u]" : "",
-				extent.e_pblk);
-		else
-			fprintf(f,
-				"%s(%lld-%lld%s):%lld-%lld",
-				printed ? ", " : "",
-				extent.e_lblk,
-				extent.e_lblk + (extent.e_len - 1),
-				extent.e_flags & EXT2_EXTENT_FLAGS_UNINIT ?
-					"[u]" : "",
-				extent.e_pblk,
-				extent.e_pblk + (extent.e_len - 1));
-		printed = 1;
-	}
-	if (printed)
-		fprintf(f, "\n");
-}
 
 void internal_dump_inode(FILE *out, const char *prefix,
 			 ext2_ino_t inode_num, struct ext2_inode *inode,
@@ -734,11 +582,9 @@ void internal_dump_inode(FILE *out, const char *prefix,
 			inode->i_file_acl, LINUX_S_ISDIR(inode->i_mode) ? inode->i_dir_acl : 0,
 			inode->osd1.hurd1.h_i_translator);
 	else
-		fprintf(out, "%sFile ACL: %llu    Directory ACL: %d\n",
+		fprintf(out, "%sFile ACL: %d    Directory ACL: %d\n",
 			prefix,
-			inode->i_file_acl | ((long long)
-				(inode->osd2.linux2.l_i_file_acl_high) << 32),
-			LINUX_S_ISDIR(inode->i_mode) ? inode->i_dir_acl : 0);
+			inode->i_file_acl, LINUX_S_ISDIR(inode->i_mode) ? inode->i_dir_acl : 0);
 	if (os == EXT2_OS_LINUX)
 		fprintf(out, "%sLinks: %d   Blockcount: %llu\n",
 			prefix, inode->i_links_count,
@@ -804,13 +650,9 @@ void internal_dump_inode(FILE *out, const char *prefix,
 		}
 		fprintf(out, "%sDevice major/minor number: %02d:%02d (hex %02x:%02x)\n",
 			devnote, major, minor, major, minor);
-	} else if (do_dump_blocks) {
-		if (inode->i_flags & EXT4_EXTENTS_FL)
-			dump_extents(out, prefix, inode_num,
-				     DUMP_LEAF_EXTENTS|DUMP_NODE_EXTENTS, 0, 0);
-		else
-			dump_blocks(out, prefix, inode_num);
 	}
+	else if (do_dump_blocks)
+		dump_blocks(out, prefix, inode_num);
 }
 
 static void dump_inode(ext2_ino_t inode_num, struct ext2_inode *inode)
@@ -853,69 +695,6 @@ void do_stat(int argc, char *argv[])
 	return;
 }
 
-void do_dump_extents(int argc, char *argv[])
-{
-	struct ext2_inode inode;
-	ext2_ino_t	ino;
-	FILE		*out;
-	int		c, flags = 0;
-	int		logical_width;
-	int		physical_width;
-
-	reset_getopt();
-	while ((c = getopt(argc, argv, "nl")) != EOF) {
-		switch (c) {
-		case 'n':
-			flags |= DUMP_NODE_EXTENTS;
-			break;
-		case 'l':
-			flags |= DUMP_LEAF_EXTENTS;
-			break;
-		}
-	}
-
-	if (argc != optind+1) {
-	print_usage:
-		com_err(0, 0, "Usage: dump_extents [-n] [-l] file");
-		return;
-	}
-
-	if (flags == 0)
-		flags = DUMP_NODE_EXTENTS | DUMP_LEAF_EXTENTS;
-	flags |= DUMP_EXTENT_TABLE;
-
-	if (check_fs_open(argv[0]))
-		return;
-
-	ino = string_to_inode(argv[optind]);
-	if (ino == 0)
-		return;
-
-	if (debugfs_read_inode(ino, &inode, argv[0]))
-		return;
-
-	if ((inode.i_flags & EXT4_EXTENTS_FL) == 0) {
-		fprintf(stderr, "%s: does not uses extent block maps\n",
-			argv[optind]);
-		return;
-	}
-
-	logical_width = int_log10(((inode.i_size |
-				    (__u64) inode.i_size_high << 32) +
-				   current_fs->blocksize - 1) /
-				  current_fs->blocksize) + 1;
-	if (logical_width < 5)
-		logical_width = 5;
-	physical_width = int_log10(current_fs->super->s_blocks_count) + 1;
-	if (physical_width < 5)
-		physical_width = 5;
-
-	out = open_pager();
-	dump_extents(out, "", ino, flags, logical_width, physical_width);
-	close_pager(out);
-	return;
-}
-
 void do_chroot(int argc, char *argv[])
 {
 	ext2_ino_t inode;
@@ -2071,7 +1850,7 @@ static int find_supp_feature(__u32 *supp, int feature_type, char *name)
 
 void do_supported_features(int argc, char *argv[])
 {
-        int	ret;
+        int	i, j, ret;
 	__u32	supp[3] = { EXT2_LIB_FEATURE_COMPAT_SUPP,
 			    EXT2_LIB_FEATURE_INCOMPAT_SUPP,
 			    EXT2_LIB_FEATURE_RO_COMPAT_SUPP };
@@ -2133,8 +1912,6 @@ static int source_file(const char *cmd_file, int sci_idx)
 			exit_status++;
 		}
 	}
-	if (f != stdin)
-		fclose(f);
 	return exit_status;
 }
 
@@ -2160,7 +1937,7 @@ int main(int argc, char **argv)
 	fprintf (stderr, "%s %s (%s)\n", debug_prog_name,
 		 E2FSPROGS_VERSION, E2FSPROGS_DATE);
 
-	while ((c = getopt (argc, argv, "iwcR:f:b:s:Vd:D")) != EOF) {
+	while ((c = getopt (argc, argv, "iwcR:f:b:s:Vd:")) != EOF) {
 		switch (c) {
 		case 'R':
 			request = optarg;
@@ -2177,9 +1954,6 @@ int main(int argc, char **argv)
 		case 'w':
 			open_flags |= EXT2_FLAG_RW;
 			break;
-		case 'D':
-			open_flags |= EXT2_FLAG_DIRECT_IO;
-			break;
 		case 'b':
 			blocksize = parse_ulong(optarg, argv[0],
 						"block size", 0);
@@ -2238,8 +2012,6 @@ int main(int argc, char **argv)
 		ss_listen(sci_idx);
 	}
 
-	ss_delete_invocation(sci_idx);
-
 	if (current_fs)
 		close_filesystem();
 
