@@ -1,10 +1,12 @@
 /*
  * dump.c --- dump the contents of an inode out to a file
- * 
+ *
  * Copyright (C) 1994 Theodore Ts'o.  This file may be redistributed
  * under the terms of the GNU Public License.
  */
 
+#define _GNU_SOURCE /* for O_LARGEFILE */
+
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -20,7 +22,7 @@
 #include <utime.h>
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
-#else 
+#else
 extern int optind;
 extern char *optarg;
 #endif
@@ -49,7 +51,7 @@ static struct {
 	{ LINUX_S_IXOTH, S_IXOTH },
 	{ 0, 0 }
 };
- 
+
 static mode_t mode_xlate(__u16 lmode)
 {
 	mode_t	mode = 0;
@@ -104,7 +106,7 @@ static void dump_file(const char *cmdname, ext2_ino_t ino, int fd,
 	ext2_file_t	e2_file;
 	int		nbytes;
 	unsigned int	got;
-	
+
 	if (debugfs_read_inode(ino, &inode, cmdname))
 		return;
 
@@ -115,12 +117,12 @@ static void dump_file(const char *cmdname, ext2_ino_t ino, int fd,
 	}
 	while (1) {
 		retval = ext2fs_file_read(e2_file, buf, sizeof(buf), &got);
-		if (retval) 
+		if (retval)
 			com_err(cmdname, retval, "while reading ext2 file");
 		if (got == 0)
 			break;
 		nbytes = write(fd, buf, got);
-		if (nbytes != got)
+		if ((unsigned) nbytes != got)
 			com_err(cmdname, errno, "while writing file");
 	}
 	retval = ext2fs_file_close(e2_file);
@@ -128,12 +130,12 @@ static void dump_file(const char *cmdname, ext2_ino_t ino, int fd,
 		com_err(cmdname, retval, "while closing ext2 file");
 		return;
 	}
-		
+
 	if (preserve)
 		fix_perms("dump_file", &inode, fd, outname);
 	else if (fd != 1)
 		close(fd);
-				    
+
 	return;
 }
 
@@ -145,7 +147,7 @@ void do_dump(int argc, char **argv)
 	int		preserve = 0;
 	const char *dump_usage = "Usage: dump_inode [-p] <file> <output_file>";
 	char		*in_fn, *out_fn;
-	
+
 	reset_getopt();
 	while ((c = getopt (argc, argv, "p")) != EOF) {
 		switch (c) {
@@ -169,7 +171,7 @@ void do_dump(int argc, char **argv)
 	out_fn = argv[optind+1];
 
 	inode = string_to_inode(in_fn);
-	if (!inode) 
+	if (!inode)
 		return;
 
 	fd = open(out_fn, O_CREAT | O_WRONLY | O_TRUNC | O_LARGEFILE, 0666);
@@ -256,7 +258,7 @@ static void rdump_inode(ext2_ino_t ino, struct ext2_inode *inode,
 		rdump_symlink(ino, inode, fullname);
 	else if (LINUX_S_ISREG(inode->i_mode)) {
 		int fd;
-		fd = open(fullname, O_WRONLY | O_CREAT | O_TRUNC, S_IRWXU);
+		fd = open(fullname, O_WRONLY | O_CREAT | O_TRUNC | O_LARGEFILE, S_IRWXU);
 		if (fd == -1) {
 			com_err("rdump", errno, "while dumping %s", fullname);
 			goto errout;
@@ -287,10 +289,12 @@ errout:
 	free(fullname);
 }
 
-static int rdump_dirent(struct ext2_dir_entry *dirent, int offset,
-			 int blocksize, char *buf, void *private)
+static int rdump_dirent(struct ext2_dir_entry *dirent,
+			int offset EXT2FS_ATTR((unused)),
+			int blocksize EXT2FS_ATTR((unused)),
+			char *buf EXT2FS_ATTR((unused)), void *private)
 {
-	char name[EXT2_NAME_LEN];
+	char name[EXT2_NAME_LEN + 1];
 	int thislen;
 	const char *dumproot = private;
 	struct ext2_inode inode;
@@ -356,7 +360,7 @@ void do_cat(int argc, char **argv)
 
 	fflush(stdout);
 	fflush(stderr);
-	dump_file(argv[0], inode, 1, 0, argv[2]); 
+	dump_file(argv[0], inode, 1, 0, argv[2]);
 
 	return;
 }
