@@ -32,16 +32,12 @@ errcode_t e2fsck_allocate_context(e2fsck_t *ret)
 	context->process_inode_size = 256;
 	context->ext_attr_ver = 2;
 	context->blocks_per_page = 1;
-	context->htree_slack_percentage = 255;
 
 	time_env = getenv("E2FSCK_TIME");
 	if (time_env)
 		context->now = strtoul(time_env, NULL, 0);
-	else {
+	else
 		context->now = time(0);
-		if (context->now < 1262322000) /* January 1 2010 */
-			context->flags |= E2F_FLAG_TIME_INSANE;
-	}
 
 	*ret = context;
 	return 0;
@@ -55,7 +51,7 @@ errcode_t e2fsck_reset_context(e2fsck_t ctx)
 {
 	int	i;
 
-	ctx->flags &= E2F_RESET_FLAGS;
+	ctx->flags = 0;
 	ctx->lost_and_found = 0;
 	ctx->bad_lost_and_found = 0;
 	if (ctx->inode_used_map) {
@@ -156,7 +152,6 @@ errcode_t e2fsck_reset_context(e2fsck_t ctx)
 	ctx->fs_dind_count = 0;
 	ctx->fs_tind_count = 0;
 	ctx->fs_fragmented = 0;
-	ctx->fs_fragmented_dir = 0;
 	ctx->large_files = 0;
 
 	for (i=0; i < MAX_EXTENT_DEPTH_COUNT; i++)
