@@ -26,7 +26,9 @@
 #ifdef HAVE_SYS_MKDEV_H
 #include <sys/mkdev.h>
 #endif
+#ifdef __linux__
 #include <sys/utsname.h>
+#endif
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -47,7 +49,7 @@ static int figure_label_len(const unsigned char *label, int len)
 	return 0;
 }
 
-static unsigned char *get_buffer(struct blkid_probe *pr, 
+static unsigned char *get_buffer(struct blkid_probe *pr,
 			  blkid_loff_t off, size_t len)
 {
 	ssize_t		ret_read;
@@ -96,7 +98,7 @@ static int check_mdraid(int fd, unsigned char *ret_uuid)
 	struct mdp_superblock_s *md;
 	blkid_loff_t		offset;
 	char			buf[4096];
-	
+
 	if (fd < 0)
 		return -BLKID_ERR_PARAM;
 
@@ -139,7 +141,7 @@ static void get_ext2_info(blkid_dev dev, struct blkid_magic *id,
 	struct ext2_super_block *es = (struct ext2_super_block *) buf;
 	const char *label = 0;
 
-	DBG(DEBUG_PROBE, printf("ext2_sb.compat = %08X:%08X:%08X\n", 
+	DBG(DEBUG_PROBE, printf("ext2_sb.compat = %08X:%08X:%08X\n",
 		   blkid_le32(es->s_feature_compat),
 		   blkid_le32(es->s_feature_incompat),
 		   blkid_le32(es->s_feature_ro_compat)));
@@ -203,6 +205,7 @@ static int fs_proc_check(const char *fs_name)
  */
 static int check_for_modules(const char *fs_name)
 {
+#ifdef __linux__
 	struct utsname	uts;
 	FILE		*f;
 	char		buf[1024], *cp, *t;
@@ -230,10 +233,13 @@ static int check_for_modules(const char *fs_name)
 			if (!strcmp(t, ".ko"))
 				*t = 0;
 		}
-		if (!strcmp(cp, fs_name))
+		if (!strcmp(cp, fs_name)) {
+			fclose(f);
 			return (1);
+		}
 	}
 	fclose(f);
+#endif
 	return (0);
 }
 
@@ -243,7 +249,7 @@ static int system_supports_ext4(void)
 	static int	ret = -1;
 	time_t		now = time(0);
 
-	if (ret != -1 || (last_check - now) < 5)
+	if (ret != -1 || (now - last_check) < 5)
 		return ret;
 	last_check = now;
 	ret = (fs_proc_check("ext4") || check_for_modules("ext4"));
@@ -256,7 +262,7 @@ static int system_supports_ext4dev(void)
 	static int	ret = -1;
 	time_t		now = time(0);
 
-	if (ret != -1 || (last_check - now) < 5)
+	if (ret != -1 || (now - last_check) < 5)
 		return ret;
 	last_check = now;
 	ret = (fs_proc_check("ext4dev") || check_for_modules("ext4dev"));
@@ -307,7 +313,7 @@ static int probe_ext4(struct blkid_probe *probe, struct blkid_magic *id,
 	es = (struct ext2_super_block *)buf;
 
 	/* Distinguish from jbd */
-	if (blkid_le32(es->s_feature_incompat) & 
+	if (blkid_le32(es->s_feature_incompat) &
 	    EXT3_FEATURE_INCOMPAT_JOURNAL_DEV)
 		return -BLKID_ERR_PARAM;
 
@@ -419,13 +425,13 @@ static unsigned char *search_fat_label(struct vfat_dir_entry *dir, int count)
 	for (i = 0; i < count; i++) {
 		if (dir[i].name[0] == 0x00)
 			break;
-		
+
 		if ((dir[i].name[0] == FAT_ENTRY_FREE) ||
 		    (dir[i].cluster_high != 0 || dir[i].cluster_low != 0) ||
 		    ((dir[i].attr & FAT_ATTR_MASK) == FAT_ATTR_LONG_NAME))
 			continue;
 
-		if ((dir[i].attr & (FAT_ATTR_VOLUME_ID | FAT_ATTR_DIR)) == 
+		if ((dir[i].attr & (FAT_ATTR_VOLUME_ID | FAT_ATTR_DIR)) ==
 		    FAT_ATTR_VOLUME_ID) {
 			return dir[i].name;
 		}
@@ -436,7 +442,7 @@ static unsigned char *search_fat_label(struct vfat_dir_entry *dir, int count)
 /* FAT label extraction from the root directory taken from Kay
  * Sievers's volume_id library */
 static int probe_fat(struct blkid_probe *probe,
-		      struct blkid_magic *id __BLKID_ATTR((unused)), 
+		      struct blkid_magic *id __BLKID_ATTR((unused)),
 		      unsigned char *buf)
 {
 	struct vfat_super_block *vs = (struct vfat_super_block *) buf;
@@ -484,11 +490,11 @@ static int probe_fat(struct blkid_probe *probe,
 	if (ms->ms_fat_length) {
 		/* the label may be an attribute in the root directory */
 		root_start = (reserved + fat_size) * sector_size;
-		root_dir_entries = vs->vs_dir_entries[0] + 
+		root_dir_entries = vs->vs_dir_entries[0] +
 			(vs->vs_dir_entries[1] << 8);
 
 		buf_size = root_dir_entries * sizeof(struct vfat_dir_entry);
-		dir = (struct vfat_dir_entry *) get_buffer(probe, root_start, 
+		dir = (struct vfat_dir_entry *) get_buffer(probe, root_start,
 							   buf_size);
 		if (dir)
 			vol_label = search_fat_label(dir, root_dir_entries);
@@ -497,7 +503,7 @@ static int probe_fat(struct blkid_probe *probe,
 			vol_label = ms->ms_label;
 		vol_serno = ms->ms_serno;
 
-		blkid_set_tag(probe->dev, "SEC_TYPE", "msdos", 
+		blkid_set_tag(probe->dev, "SEC_TYPE", "msdos",
 			      sizeof("msdos"));
 	} else {
 		/* Search the FAT32 root dir for the label attribute */
@@ -511,10 +517,10 @@ static int probe_fat(struct blkid_probe *probe,
 			int count;
 
 			next_sect_off = (next - 2) * vs->vs_cluster_size;
-			next_off = (start_data_sect + next_sect_off) * 
+			next_off = (start_data_sect + next_sect_off) *
 				sector_size;
 
-			dir = (struct vfat_dir_entry *) 
+			dir = (struct vfat_dir_entry *)
 				get_buffer(probe, next_off, buf_size);
 			if (dir == NULL)
 				break;
@@ -526,7 +532,7 @@ static int probe_fat(struct blkid_probe *probe,
 				break;
 
 			/* get FAT entry */
-			fat_entry_off = (reserved * sector_size) + 
+			fat_entry_off = (reserved * sector_size) +
 				(next * sizeof(__u32));
 			buf = get_buffer(probe, fat_entry_off, buf_size);
 			if (buf == NULL)
@@ -563,35 +569,48 @@ static int probe_fat(struct blkid_probe *probe,
  * [7-Jul-2005, Karel Zak <kzak@redhat.com>]
  */
 static int probe_fat_nomagic(struct blkid_probe *probe,
-			     struct blkid_magic *id __BLKID_ATTR((unused)), 
+			     struct blkid_magic *id __BLKID_ATTR((unused)),
 			     unsigned char *buf)
 {
-	struct vfat_super_block *vs;
+	struct msdos_super_block *ms;
 
-	vs = (struct vfat_super_block *)buf;
+	ms = (struct msdos_super_block *)buf;
 
 	/* heads check */
-	if (vs->vs_heads == 0)
+	if (ms->ms_heads == 0)
 		return 1;
 
-	/* cluster size check*/	
-	if (vs->vs_cluster_size == 0 ||
-	    (vs->vs_cluster_size & (vs->vs_cluster_size-1)))
+	/* cluster size check*/
+	if (ms->ms_cluster_size == 0 ||
+	    (ms->ms_cluster_size & (ms->ms_cluster_size-1)))
 		return 1;
 
 	/* media check */
-	if (vs->vs_media < 0xf8 && vs->vs_media != 0xf0)
+	if (ms->ms_media < 0xf8 && ms->ms_media != 0xf0)
 		return 1;
 
 	/* fat counts(Linux kernel expects at least 1 FAT table) */
-	if (!vs->vs_fats)
+	if (!ms->ms_fats)
+		return 1;
+
+	/*
+	 * OS/2 and apparently DFSee will place a FAT12/16-like
+	 * pseudo-superblock in the first 512 bytes of non-FAT
+	 * filesystems --- at least JFS and HPFS, and possibly others.
+	 * So we explicitly check for those filesystems at the
+	 * FAT12/16 filesystem magic field identifier, and if they are
+	 * present, we rule this out as a FAT filesystem, despite the
+	 * FAT-like pseudo-header.
+         */
+	if ((memcmp(ms->ms_magic, "JFS     ", 8) == 0) ||
+	    (memcmp(ms->ms_magic, "HPFS    ", 8) == 0))
 		return 1;
 
 	return probe_fat(probe, id, buf);
 }
 
 static int probe_ntfs(struct blkid_probe *probe,
-		      struct blkid_magic *id __BLKID_ATTR((unused)), 
+		      struct blkid_magic *id __BLKID_ATTR((unused)),
 		      unsigned char *buf)
 {
 	struct ntfs_super_block *ns;
@@ -618,7 +637,7 @@ static int probe_ntfs(struct blkid_probe *probe,
 	if (ns->cluster_per_mft_record < 0)
 		mft_record_size = 1 << (0-ns->cluster_per_mft_record);
 	else
-		mft_record_size = ns->cluster_per_mft_record * 
+		mft_record_size = ns->cluster_per_mft_record *
 			sectors_per_cluster * bytes_per_sector;
 	nr_clusters = blkid_le64(ns->number_of_sectors) / sectors_per_cluster;
 
@@ -626,7 +645,7 @@ static int probe_ntfs(struct blkid_probe *probe,
 	    (blkid_le64(ns->mft_mirror_cluster_location) > nr_clusters))
 		return 1;
 
-	off = blkid_le64(ns->mft_mirror_cluster_location) * 
+	off = blkid_le64(ns->mft_mirror_cluster_location) *
 		bytes_per_sector * sectors_per_cluster;
 
 	buf_mft = get_buffer(probe, off, mft_record_size);
@@ -636,7 +655,7 @@ static int probe_ntfs(struct blkid_probe *probe,
 	if (memcmp(buf_mft, "FILE", 4))
 		return 1;
 
-	off = blkid_le64(ns->mft_cluster_location) * bytes_per_sector * 
+	off = blkid_le64(ns->mft_cluster_location) * bytes_per_sector *
 		sectors_per_cluster;
 
 	buf_mft = get_buffer(probe, off, mft_record_size);
@@ -659,7 +678,7 @@ static int probe_ntfs(struct blkid_probe *probe,
 
 	attr_off = blkid_le16(mft->attrs_offset);
 	label_str[0] = 0;
-	
+
 	while (1) {
 		attr = (struct file_attribute *) (buf_mft + attr_off);
 		attr_len = blkid_le16(attr->len);
@@ -699,7 +718,7 @@ static int probe_ntfs(struct blkid_probe *probe,
 
 
 static int probe_xfs(struct blkid_probe *probe,
-		     struct blkid_magic *id __BLKID_ATTR((unused)), 
+		     struct blkid_magic *id __BLKID_ATTR((unused)),
 		     unsigned char *buf)
 {
 	struct xfs_super_block *xs;
@@ -743,7 +762,7 @@ static int probe_reiserfs(struct blkid_probe *probe,
 }
 
 static int probe_reiserfs4(struct blkid_probe *probe,
-			   struct blkid_magic *id __BLKID_ATTR((unused)), 
+			   struct blkid_magic *id __BLKID_ATTR((unused)),
 			   unsigned char *buf)
 {
 	struct reiser4_super_block *rs4 = (struct reiser4_super_block *) buf;
@@ -752,14 +771,14 @@ static int probe_reiserfs4(struct blkid_probe *probe,
 	if (strlen((char *) rs4->rs4_label))
 		label = rs4->rs4_label;
 	set_uuid(probe->dev, rs4->rs4_uuid, 0);
-	blkid_set_tag(probe->dev, "LABEL", (const char *) label, 
+	blkid_set_tag(probe->dev, "LABEL", (const char *) label,
 		      sizeof(rs4->rs4_label));
 
 	return 0;
 }
 
 static int probe_jfs(struct blkid_probe *probe,
-		     struct blkid_magic *id __BLKID_ATTR((unused)), 
+		     struct blkid_magic *id __BLKID_ATTR((unused)),
 		     unsigned char *buf)
 {
 	struct jfs_super_block *js;
@@ -767,6 +786,16 @@ static int probe_jfs(struct blkid_probe *probe,
 
 	js = (struct jfs_super_block *)buf;
 
+	if (blkid_le32(js->js_bsize) != (1 << blkid_le16(js->js_l2bsize)))
+		return 1;
+
+	if (blkid_le32(js->js_pbsize) != (1 << blkid_le16(js->js_l2pbsize)))
+		return 1;
+
+	if ((blkid_le16(js->js_l2bsize) - blkid_le16(js->js_l2pbsize)) !=
+	    blkid_le16(js->js_l2bfactor))
+		return 1;
+
 	if (strlen((char *) js->js_label))
 		label = (char *) js->js_label;
 	blkid_set_tag(probe->dev, "LABEL", label, sizeof(js->js_label));
@@ -802,7 +831,7 @@ static int probe_luks(struct blkid_probe *probe,
 }
 
 static int probe_romfs(struct blkid_probe *probe,
-		       struct blkid_magic *id __BLKID_ATTR((unused)), 
+		       struct blkid_magic *id __BLKID_ATTR((unused)),
 		       unsigned char *buf)
 {
 	struct romfs_super_block *ros;
@@ -817,7 +846,7 @@ static int probe_romfs(struct blkid_probe *probe,
 }
 
 static int probe_cramfs(struct blkid_probe *probe,
-			struct blkid_magic *id __BLKID_ATTR((unused)), 
+			struct blkid_magic *id __BLKID_ATTR((unused)),
 			unsigned char *buf)
 {
 	struct cramfs_super_block *csb;
@@ -865,7 +894,7 @@ static int probe_swap1(struct blkid_probe *probe,
 	/* arbitrary sanity check.. is there any garbage down there? */
 	if (sws->sws_pad[32] == 0 && sws->sws_pad[33] == 0)  {
 		if (sws->sws_volume[0])
-			blkid_set_tag(probe->dev, "LABEL", sws->sws_volume, 
+			blkid_set_tag(probe->dev, "LABEL", sws->sws_volume,
 				      sizeof(sws->sws_volume));
 		if (sws->sws_uuid[0])
 			set_uuid(probe->dev, sws->sws_uuid, 0);
@@ -874,7 +903,7 @@ static int probe_swap1(struct blkid_probe *probe,
 }
 
 static int probe_iso9660(struct blkid_probe *probe,
-			 struct blkid_magic *id __BLKID_ATTR((unused)), 
+			 struct blkid_magic *id __BLKID_ATTR((unused)),
 			 unsigned char *buf)
 {
 	struct iso_volume_descriptor *iso;
@@ -883,7 +912,7 @@ static int probe_iso9660(struct blkid_probe *probe,
 	iso = (struct iso_volume_descriptor *) buf;
 	label = iso->volume_id;
 
-	blkid_set_tag(probe->dev, "LABEL", (const char *) label, 
+	blkid_set_tag(probe->dev, "LABEL", (const char *) label,
 		      figure_label_len(label, 32));
 	return 0;
 }
@@ -894,7 +923,7 @@ static const char
 		 "NSR03", "TEA01", 0 };
 
 static int probe_udf(struct blkid_probe *probe,
-		     struct blkid_magic *id __BLKID_ATTR((unused)), 
+		     struct blkid_magic *id __BLKID_ATTR((unused)),
 		     unsigned char *buf __BLKID_ATTR((unused)))
 {
 	int j, bs;
@@ -904,7 +933,7 @@ static int probe_udf(struct blkid_probe *probe,
 	/* determine the block size by scanning in 2K increments
 	   (block sizes larger than 2K will be null padded) */
 	for (bs = 1; bs < 16; bs++) {
-		isosb = (struct iso_volume_descriptor *) 
+		isosb = (struct iso_volume_descriptor *)
 			get_buffer(probe, bs*2048+32768, sizeof(isosb));
 		if (!isosb)
 			return 1;
@@ -915,8 +944,8 @@ static int probe_udf(struct blkid_probe *probe,
 	/* Scan up to another 64 blocks looking for additional VSD's */
 	for (j = 1; j < 64; j++) {
 		if (j > 1) {
-			isosb = (struct iso_volume_descriptor *) 
-				get_buffer(probe, j*bs*2048+32768, 
+			isosb = (struct iso_volume_descriptor *)
+				get_buffer(probe, j*bs*2048+32768,
 					   sizeof(isosb));
 			if (!isosb)
 				return 1;
@@ -937,7 +966,7 @@ static int probe_udf(struct blkid_probe *probe,
 }
 
 static int probe_ocfs(struct blkid_probe *probe,
-		      struct blkid_magic *id __BLKID_ATTR((unused)), 
+		      struct blkid_magic *id __BLKID_ATTR((unused)),
 		      unsigned char *buf)
 {
 	struct ocfs_volume_header ovh;
@@ -952,7 +981,7 @@ static int probe_ocfs(struct blkid_probe *probe,
 		blkid_set_tag(probe->dev,"SEC_TYPE","ocfs1",sizeof("ocfs1"));
 	else if (major >= 9)
 		blkid_set_tag(probe->dev,"SEC_TYPE","ntocfs",sizeof("ntocfs"));
-	
+
 	blkid_set_tag(probe->dev, "LABEL", ovl.label, ocfslabellen(ovl));
 	blkid_set_tag(probe->dev, "MOUNT", ovh.mount, ocfsmountlen(ovh));
 	set_uuid(probe->dev, ovl.vol_id, 0);
@@ -960,7 +989,7 @@ static int probe_ocfs(struct blkid_probe *probe,
 }
 
 static int probe_ocfs2(struct blkid_probe *probe,
-		       struct blkid_magic *id __BLKID_ATTR((unused)), 
+		       struct blkid_magic *id __BLKID_ATTR((unused)),
 		       unsigned char *buf)
 {
 	struct ocfs2_super_block *osb;
@@ -973,7 +1002,7 @@ static int probe_ocfs2(struct blkid_probe *probe,
 }
 
 static int probe_oracleasm(struct blkid_probe *probe,
-			   struct blkid_magic *id __BLKID_ATTR((unused)), 
+			   struct blkid_magic *id __BLKID_ATTR((unused)),
 			   unsigned char *buf)
 {
 	struct oracle_asm_disk_label *dl;
@@ -995,9 +1024,9 @@ static int probe_gfs(struct blkid_probe *probe,
 
 	if (blkid_be32(sbd->sb_fs_format) == GFS_FORMAT_FS &&
 	    blkid_be32(sbd->sb_multihost_format) == GFS_FORMAT_MULTI)
-	{	
+	{
 		blkid_set_tag(probe->dev, "UUID", 0, 0);
-	
+
 		if (strlen(sbd->sb_locktable))
 			label = sbd->sb_locktable;
 		blkid_set_tag(probe->dev, "LABEL", label, sizeof(sbd->sb_locktable));
@@ -1017,9 +1046,9 @@ static int probe_gfs2(struct blkid_probe *probe,
 
 	if (blkid_be32(sbd->sb_fs_format) == GFS2_FORMAT_FS &&
 	    blkid_be32(sbd->sb_multihost_format) == GFS2_FORMAT_MULTI)
-	{	
+	{
 		blkid_set_tag(probe->dev, "UUID", 0, 0);
-	
+
 		if (strlen(sbd->sb_locktable))
 			label = sbd->sb_locktable;
 		blkid_set_tag(probe->dev, "LABEL", label, sizeof(sbd->sb_locktable));
@@ -1028,17 +1057,178 @@ static int probe_gfs2(struct blkid_probe *probe,
 	return 1;
 }
 
-static int probe_hfsplus(struct blkid_probe *probe __BLKID_ATTR((unused)),
+static void unicode_16be_to_utf8(unsigned char *str, int out_len,
+				 const unsigned char *buf, int in_len)
+{
+	int i, j;
+	unsigned int c;
+
+	for (i = j = 0; i + 2 <= in_len; i += 2) {
+		c = (buf[i] << 8) | buf[i+1];
+		if (c == 0) {
+			str[j] = '\0';
+			break;
+		} else if (c < 0x80) {
+			if (j+1 >= out_len)
+				break;
+			str[j++] = (unsigned char) c;
+		} else if (c < 0x800) {
+			if (j+2 >= out_len)
+				break;
+			str[j++] = (unsigned char) (0xc0 | (c >> 6));
+			str[j++] = (unsigned char) (0x80 | (c & 0x3f));
+		} else {
+			if (j+3 >= out_len)
+				break;
+			str[j++] = (unsigned char) (0xe0 | (c >> 12));
+			str[j++] = (unsigned char) (0x80 | ((c >> 6) & 0x3f));
+			str[j++] = (unsigned char) (0x80 | (c & 0x3f));
+		}
+	}
+	str[j] = '\0';
+}
+
+static int probe_hfs(struct blkid_probe *probe __BLKID_ATTR((unused)),
 			 struct blkid_magic *id __BLKID_ATTR((unused)),
 			 unsigned char *buf)
 {
-	struct hfs_mdb *sbd = (struct hfs_mdb *)buf;
+	struct hfs_mdb *hfs = (struct hfs_mdb *) buf;
+	char	uuid_str[17];
+	__u64	uuid;
+
+	if ((memcmp(hfs->embed_sig, "H+", 2) == 0) ||
+	    (memcmp(hfs->embed_sig, "HX", 2) == 0))
+		return 1;	/* Not hfs, but an embedded HFS+ */
+
+	uuid = blkid_le64(*((unsigned long long *) hfs->finder_info.id));
+	if (uuid) {
+		sprintf(uuid_str, "%016llX", uuid);
+		blkid_set_tag(probe->dev, "UUID", uuid_str, 0);
+	}
+	blkid_set_tag(probe->dev, "LABEL", hfs->label, hfs->label_len);
+	return 0;
+}
+
+
+static int probe_hfsplus(struct blkid_probe *probe,
+			 struct blkid_magic *id,
+			 unsigned char *buf)
+{
+	struct hfsplus_extent extents[HFSPLUS_EXTENT_COUNT];
+	struct hfsplus_bnode_descriptor *descr;
+	struct hfsplus_bheader_record *bnode;
+	struct hfsplus_catalog_key *key;
+	struct hfsplus_vol_header *hfsplus;
+	struct hfs_mdb *sbd = (struct hfs_mdb *) buf;
+	unsigned int alloc_block_size;
+	unsigned int alloc_first_block;
+	unsigned int embed_first_block;
+	unsigned int off = 0;
+	unsigned int blocksize;
+	unsigned int cat_block;
+	unsigned int ext_block_start;
+	unsigned int ext_block_count;
+	unsigned int record_count;
+	unsigned int leaf_node_head;
+	unsigned int leaf_node_count;
+	unsigned int leaf_node_size;
+	unsigned int leaf_block;
+	unsigned int label_len;
+	int ext;
+	__u64 leaf_off, uuid;
+	char	uuid_str[17], label[512];
 
 	/* Check for a HFS+ volume embedded in a HFS volume */
-	if (memcmp(sbd->embed_sig, "H+", 2) == 0)
+	if (memcmp(sbd->signature, "BD", 2) == 0) {
+		if ((memcmp(sbd->embed_sig, "H+", 2) != 0) &&
+		    (memcmp(sbd->embed_sig, "HX", 2) != 0))
+			/* This must be an HFS volume, so fail */
+			return 1;
+
+		alloc_block_size = blkid_be32(sbd->al_blk_size);
+		alloc_first_block = blkid_be16(sbd->al_bl_st);
+		embed_first_block = blkid_be16(sbd->embed_startblock);
+		off = (alloc_first_block * 512) +
+			(embed_first_block * alloc_block_size);
+		buf = get_buffer(probe, off + (id->bim_kboff * 1024),
+				 sizeof(sbd));
+		if (!buf)
+			return 1;
+
+		hfsplus = (struct hfsplus_vol_header *) buf;
+	}
+
+	hfsplus = (struct hfsplus_vol_header *) buf;
+
+	if ((memcmp(hfsplus->signature, "H+", 2) != 0) &&
+	    (memcmp(hfsplus->signature, "HX", 2) != 0))
+		return 1;
+
+	uuid = blkid_le64(*((unsigned long long *) hfsplus->finder_info.id));
+	if (uuid) {
+		sprintf(uuid_str, "%016llX", uuid);
+		blkid_set_tag(probe->dev, "UUID", uuid_str, 0);
+	}
+
+	blocksize = blkid_be32(hfsplus->blocksize);
+	memcpy(extents, hfsplus->cat_file.extents, sizeof(extents));
+	cat_block = blkid_be32(extents[0].start_block);
+
+	buf = get_buffer(probe, off + (cat_block * blocksize), 0x2000);
+	if (!buf)
 		return 0;
 
-	return 1;
+	bnode = (struct hfsplus_bheader_record *)
+		&buf[sizeof(struct hfsplus_bnode_descriptor)];
+
+	leaf_node_head = blkid_be32(bnode->leaf_head);
+	leaf_node_size = blkid_be16(bnode->node_size);
+	leaf_node_count = blkid_be32(bnode->leaf_count);
+	if (leaf_node_count == 0)
+		return 0;
+
+	leaf_block = (leaf_node_head * leaf_node_size) / blocksize;
+
+	/* get physical location */
+	for (ext = 0; ext < HFSPLUS_EXTENT_COUNT; ext++) {
+		ext_block_start = blkid_be32(extents[ext].start_block);
+		ext_block_count = blkid_be32(extents[ext].block_count);
+		if (ext_block_count == 0)
+			return 0;
+
+		/* this is our extent */
+		if (leaf_block < ext_block_count)
+			break;
+
+		leaf_block -= ext_block_count;
+	}
+	if (ext == HFSPLUS_EXTENT_COUNT)
+		return 0;
+
+	leaf_off = (ext_block_start + leaf_block) * blocksize;
+
+	buf = get_buffer(probe, off + leaf_off, leaf_node_size);
+	if (!buf)
+		return 0;
+
+	descr = (struct hfsplus_bnode_descriptor *) buf;
+	record_count = blkid_be16(descr->num_recs);
+	if (record_count == 0)
+		return 0;
+
+	if (descr->type != HFS_NODE_LEAF)
+		return 0;
+
+	key = (struct hfsplus_catalog_key *)
+		&buf[sizeof(struct hfsplus_bnode_descriptor)];
+
+	if (blkid_be32(key->parent_id) != HFSPLUS_POR_CNID)
+		return 0;
+
+	label_len = blkid_be16(key->unicode_len) * 2;
+	unicode_16be_to_utf8(label, sizeof(label), key->unicode, label_len);
+	blkid_set_tag(probe->dev, "LABEL", label, 0);
+	return 0;
 }
 
 #define LVM2_LABEL_SIZE 512
@@ -1162,8 +1352,9 @@ static struct blkid_magic type_array[] = {
   { "zfs",     264,	 0,  8, "\0\0\x02\xf5\xb0\x07\xb1\x0c", probe_zfs },
   { "zfs",     264,	 0,  8, "\x0c\xb1\x07\xb0\xf5\x02\0\0", probe_zfs },
   { "hfsplus",	 1,	 0,  2, "BD",			probe_hfsplus },
-  { "hfsplus",	 1,	 0,  2, "H+",			0 },
-  { "hfs",	 1,	 0,  2, "BD",			0 },
+  { "hfsplus",	 1,	 0,  2, "H+",			probe_hfsplus },
+  { "hfsplus",	 1,	 0,  2, "HX",			probe_hfsplus },
+  { "hfs",	 1,	 0,  2, "BD",			probe_hfs },
   { "ufs",	 8,  0x55c,  4, "T\031\001\000",	0 },
   { "hpfs",	 8,	 0,  4, "I\350\225\371",	0 },
   { "sysv",	 0,  0x3f8,  4, "\020~\030\375",	0 },
@@ -1246,7 +1437,7 @@ blkid_dev blkid_verify(blkid_cache cache, blkid_dev dev)
 
 	if ((now >= dev->bid_time) &&
 	    (st.st_mtime <= dev->bid_time) &&
-	    ((diff < BLKID_PROBE_MIN) || 
+	    ((diff < BLKID_PROBE_MIN) ||
 	     (dev->bid_flags & BLKID_BID_FL_VERIFIED &&
 	      diff < BLKID_PROBE_INTERVAL)))
 		return dev;
@@ -1259,7 +1450,7 @@ blkid_dev blkid_verify(blkid_cache cache, blkid_dev dev)
 
 	if ((probe.fd = open(dev->bid_name, O_RDONLY)) < 0) {
 		DBG(DEBUG_PROBE, printf("blkid_verify: error %s (%d) while "
-					"opening %s\n", strerror(errno), errno, 
+					"opening %s\n", strerror(errno), errno,
 					dev->bid_name));
 		goto open_err;
 	}
@@ -1269,12 +1460,12 @@ blkid_dev blkid_verify(blkid_cache cache, blkid_dev dev)
 	probe.sbbuf = 0;
 	probe.buf = 0;
 	probe.buf_max = 0;
-	
+
 	/*
 	 * Iterate over the type array.  If we already know the type,
 	 * then try that first.  If it doesn't work, then blow away
 	 * the type information, and try again.
-	 * 
+	 *
 	 */
 try_again:
 	type = 0;
@@ -1345,7 +1536,7 @@ found_type:
 		free(probe.sbbuf);
 	if (probe.buf)
 		free(probe.buf);
-	if (probe.fd >= 0) 
+	if (probe.fd >= 0)
 		close(probe.fd);
 
 	return dev;
@@ -1389,7 +1580,7 @@ int main(int argc, char **argv)
 		printf("LABEL='%s'\n", dev->bid_label);
 	if (dev->bid_uuid)
 		printf("UUID='%s'\n", dev->bid_uuid);
-	
+
 	blkid_free_dev(dev);
 	return (0);
 }
