@@ -54,8 +54,10 @@ void blkid_free_tag(blkid_tag tag)
 	list_del(&tag->bit_tags);	/* list of tags for this device */
 	list_del(&tag->bit_names);	/* list of tags with this type */
 
-	free(tag->bit_name);
-	free(tag->bit_val);
+	if (tag->bit_name)
+		free(tag->bit_name);
+	if (tag->bit_val)
+		free(tag->bit_val);
 
 	free(tag);
 }
@@ -204,7 +206,8 @@ int blkid_set_tag(blkid_dev dev, const char *name,
 errout:
 	if (t)
 		blkid_free_tag(t);
-	else free(val);
+	else if (val)
+		free(val);
 	if (head)
 		blkid_free_tag(head);
 	return -BLKID_ERR_MEM;
