@@ -4,9 +4,10 @@
  * Copyright (C) 2001 Theodore Ts'o.
  *
  * %Begin-Header%
- * This file may be redistributed under the terms of the GNU Library
- * General Public License, version 2.
+ * This file may be redistributed under the terms of the GNU Public
+ * License.
  * %End-Header%
+ *
  */
 
 #include <stdio.h>
@@ -19,13 +20,6 @@ void ext2fs_inode_alloc_stats2(ext2_filsys fs, ext2_ino_t ino,
 {
 	int	group = ext2fs_group_of_ino(fs, ino);
 
-#ifndef OMIT_COM_ERR
-	if (ino > fs->super->s_inodes_count) {
-		com_err("ext2fs_inode_alloc_stats2", 0,
-			"Illegal inode number: %lu", (unsigned long) ino);
-		return;
-	}
-#endif
 	if (inuse > 0)
 		ext2fs_mark_inode_bitmap(fs->inode_map, ino);
 	else
@@ -64,13 +58,6 @@ void ext2fs_block_alloc_stats(ext2_filsys fs, blk_t blk, int inuse)
 {
 	int	group = ext2fs_group_of_blk(fs, blk);
 
-#ifndef OMIT_COM_ERR
-	if (blk >= fs->super->s_blocks_count) {
-		com_err("ext2fs_block_alloc_stats", 0,
-			"Illegal block number: %lu", (unsigned long) blk);
-		return;
-	}
-#endif
 	if (inuse > 0)
 		ext2fs_mark_block_bitmap(fs->block_map, blk);
 	else
