@@ -73,21 +73,31 @@ struct ext2_extent_path {
 static void dbg_show_header(struct ext3_extent_header *eh)
 {
 	printf("header: magic=%x entries=%u max=%u depth=%u generation=%u\n",
-	       eh->eh_magic, eh->eh_entries, eh->eh_max, eh->eh_depth,
-	       eh->eh_generation);
+			ext2fs_le16_to_cpu(eh->eh_magic),
+			ext2fs_le16_to_cpu(eh->eh_entries),
+			ext2fs_le16_to_cpu(eh->eh_max),
+			ext2fs_le16_to_cpu(eh->eh_depth),
+			ext2fs_le32_to_cpu(eh->eh_generation));
 }
 
 static void dbg_show_index(struct ext3_extent_idx *ix)
 {
 	printf("index: block=%u leaf=%u leaf_hi=%u unused=%u\n",
-	       ix->ei_block, ix->ei_leaf, ix->ei_leaf_hi, ix->ei_unused);
+			ext2fs_le32_to_cpu(ix->ei_block),
+			ext2fs_le32_to_cpu(ix->ei_leaf),
+			ext2fs_le16_to_cpu(ix->ei_leaf_hi),
+			ext2fs_le16_to_cpu(ix->ei_unused));
 }
 
 static void dbg_show_extent(struct ext3_extent *ex)
 {
 	printf("extent: block=%u-%u len=%u start=%u start_hi=%u\n",
-	       ex->ee_block, ex->ee_block + ex->ee_len - 1,
-	       ex->ee_len, ex->ee_start, ex->ee_start_hi);
+			ext2fs_le32_to_cpu(ex->ee_block),
+			ext2fs_le32_to_cpu(ex->ee_block) +
+			ext2fs_le16_to_cpu(ex->ee_len) - 1,
+			ext2fs_le16_to_cpu(ex->ee_len),
+			ext2fs_le32_to_cpu(ex->ee_start),
+			ext2fs_le16_to_cpu(ex->ee_start_hi));
 }
 
 static void dbg_print_extent(char *desc, struct ext2fs_extent *extent)
@@ -109,13 +119,11 @@ static void dbg_print_extent(char *desc, struct ext2fs_extent *extent)
 
 }
 
-#define dbg_printf(fmt, args...) printf(fmt, ## args)
 #else
 #define dbg_show_header(eh) do { } while (0)
 #define dbg_show_index(ix) do { } while (0)
 #define dbg_show_extent(ex) do { } while (0)
 #define dbg_print_extent(desc, ex) do { } while (0)
-#define dbg_printf(fmt, args...) do { } while (0)
 #endif
 
 /*
@@ -139,7 +147,7 @@ errcode_t ext2fs_extent_header_verify(void *ptr, int size)
 	eh_max = (size - sizeof(*eh)) / entry_size;
 	/* Allow two extent-sized items at the end of the block, for
 	 * ext4_extent_tail with checksum in the future. */
-	if ((ext2fs_le16_to_cpu(eh->eh_max) > eh_max) || 
+	if ((ext2fs_le16_to_cpu(eh->eh_max) > eh_max) ||
 	    (ext2fs_le16_to_cpu(eh->eh_max) < (eh_max - 2)))
 		return EXT2_ET_EXTENT_HEADER_BAD;
 
@@ -214,12 +222,14 @@ extern errcode_t ext2fs_extent_open(ext2_filsys fs, ext2_ino_t ino,
 		handle->inode->i_flags |= EXT4_EXTENTS_FL;
 	}
 
-	if (!(handle->inode->i_flags & EXT4_EXTENTS_FL))
-		return EXT2_ET_INODE_NOT_EXTENT;
+	if (!(handle->inode->i_flags & EXT4_EXTENTS_FL)) {
+		retval = EXT2_ET_INODE_NOT_EXTENT;
+		goto errout;
+	}
 
 	retval = ext2fs_extent_header_verify(eh, sizeof(handle->inode->i_block));
 	if (retval)
-		return (retval);
+		goto errout;
 
 	handle->max_depth = ext2fs_le16_to_cpu(eh->eh_depth);
 	handle->type = ext2fs_le16_to_cpu(eh->eh_magic);
@@ -299,9 +309,11 @@ retry:
 				return EXT2_ET_EXTENT_NO_NEXT;
 		}
 		if (op != EXT2_EXTENT_NEXT_SIB) {
-			dbg_printf("<<<< OP = %s\n",
+#ifdef DEBUG
+			printf("<<<< OP = %s\n",
 			       (op == EXT2_EXTENT_DOWN) ? "down" :
 			       ((op == EXT2_EXTENT_UP) ? "up" : "unknown"));
+#endif
 		}
 	}
 
@@ -328,9 +340,11 @@ retry:
 				return EXT2_ET_EXTENT_NO_PREV;
 		}
 		if (op != EXT2_EXTENT_PREV_SIB) {
-			dbg_printf("<<<< OP = %s\n",
+#ifdef DEBUG
+			printf("<<<< OP = %s\n",
 			       (op == EXT2_EXTENT_DOWN_AND_LAST) ? "down/last" :
 			       ((op == EXT2_EXTENT_UP) ? "up" : "unknown"));
+#endif
 		}
 	}
 
@@ -340,8 +354,10 @@ retry:
 			op = EXT2_EXTENT_DOWN;
 		else
 			op = EXT2_EXTENT_LAST_SIB;
-		dbg_printf("<<<< OP = %s\n",
+#ifdef DEBUG
+		printf("<<<< OP = %s\n",
 			   (op == EXT2_EXTENT_DOWN) ? "down" : "last_sib");
+#endif
 	}
 
 	switch (op) {
@@ -451,11 +467,11 @@ retry:
 			if (handle->level < handle->max_depth)
 				path->visit_num = 1;
 		}
-
-		dbg_printf("Down to level %d/%d, end_blk=%llu\n",
+#ifdef DEBUG
+		printf("Down to level %d/%d, end_blk=%llu\n",
 			   handle->level, handle->max_depth,
 			   path->end_blk);
-
+#endif
 		break;
 	default:
 		return EXT2_ET_OP_NOT_SUPPORTED;
@@ -465,7 +481,9 @@ retry:
 		return EXT2_ET_NO_CURRENT_NODE;
 
 	extent->e_flags = 0;
-	dbg_printf("(Left %d)\n", path->left);
+#ifdef DEBUG
+	printf("(Left %d)\n", path->left);
+#endif
 
 	if (handle->level == handle->max_depth) {
 		ex = (struct ext3_extent *) ix;
@@ -582,12 +600,17 @@ static errcode_t extent_goto(ext2_extent_handle_t handle,
 	errcode_t		retval;
 
 	retval = ext2fs_extent_get(handle, EXT2_EXTENT_ROOT, &extent);
-	if (retval)
+	if (retval) {
+		if (retval == EXT2_ET_EXTENT_NO_NEXT)
+			retval = EXT2_ET_EXTENT_NOT_FOUND;
 		return retval;
+	}
 
 	if (leaf_level > handle->max_depth) {
-		dbg_printf("leaf level %d greater than tree depth %d\n",
+#ifdef DEBUG
+		printf("leaf level %d greater than tree depth %d\n",
 			leaf_level, handle->max_depth);
+#endif
 		return EXT2_ET_OP_NOT_SUPPORTED;
 	}
 
@@ -708,7 +731,7 @@ done:
 	return retval;
 }
 
-errcode_t ext2fs_extent_replace(ext2_extent_handle_t handle, 
+errcode_t ext2fs_extent_replace(ext2_extent_handle_t handle,
 				int flags EXT2FS_ATTR((unused)),
 				struct ext2fs_extent *extent)
 {
@@ -786,8 +809,9 @@ static errcode_t extent_node_split(ext2_extent_handle_t handle)
 	if (!handle->path)
 		return EXT2_ET_NO_CURRENT_NODE;
 
-	dbg_printf("splitting node at level %d\n", handle->level);
-
+#ifdef DEBUG
+	printf("splitting node at level %d\n", handle->level);
+#endif
 	retval = ext2fs_extent_get(handle, EXT2_EXTENT_CURRENT, &extent);
 	if (retval)
 		goto done;
@@ -805,8 +829,10 @@ static errcode_t extent_node_split(ext2_extent_handle_t handle)
 			(handle->path[handle->level - 1].entries >=
 			 handle->path[handle->level - 1].max_entries)) {
 
-		dbg_printf("parent level %d full; splitting it too\n",
+#ifdef DEBUG
+		printf("parent level %d full; splitting it too\n",
 							handle->level - 1);
+#endif
 		/* split the parent */
 		retval = ext2fs_extent_get(handle, EXT2_EXTENT_UP, &extent);
 		if (retval)
@@ -846,12 +872,16 @@ static errcode_t extent_node_split(ext2_extent_handle_t handle)
 		tocopy = ext2fs_le16_to_cpu(eh->eh_entries) / 2;
 	}
 
-	dbg_printf("will copy out %d of %d entries at level %d\n",
+#ifdef DEBUG
+	printf("will copy out %d of %d entries at level %d\n",
 				tocopy, ext2fs_le16_to_cpu(eh->eh_entries),
 				handle->level);
+#endif
 
 	if (!tocopy) {
-		dbg_printf("Nothing to copy to new block!\n");
+#ifdef DEBUG
+		printf("Nothing to copy to new block!\n");
+#endif
 		retval = EXT2_ET_CANT_SPLIT_EXTENT;
 		goto done;
 	}
@@ -877,7 +907,9 @@ static errcode_t extent_node_split(ext2_extent_handle_t handle)
 	if (retval)
 		goto done;
 
-	dbg_printf("will copy to new node at block %lu\n", new_node_pblk);
+#ifdef DEBUG
+	printf("will copy to new node at block %lu\n", new_node_pblk);
+#endif
 
 	/* Copy data into new block buffer */
 	/* First the header for the new block... */
@@ -1001,8 +1033,10 @@ errcode_t ext2fs_extent_insert(ext2_extent_handle_t handle, int flags,
 		if (flags & EXT2_EXTENT_INSERT_NOSPLIT) {
 			return EXT2_ET_CANT_INSERT_EXTENT;
 		} else {
-			dbg_printf("node full (level %d) - splitting\n",
+#ifdef DEBUG
+			printf("node full (level %d) - splitting\n",
 				   handle->level);
+#endif
 			retval = extent_node_split(handle);
 			if (retval)
 				goto errout;
@@ -1126,6 +1160,7 @@ errcode_t ext2fs_extent_set_bmap(ext2_extent_handle_t handle,
 	orig_height = info.max_depth - info.curr_level;
 	orig_lblk = extent.e_lblk;
 
+again:
 	/* go to the logical spot we want to (re/un)map */
 	retval = ext2fs_extent_goto(handle, logical);
 	if (retval) {
@@ -1133,8 +1168,10 @@ errcode_t ext2fs_extent_set_bmap(ext2_extent_handle_t handle,
 			retval = 0;
 			mapped = 0;
 			if (!physical) {
-				dbg_printf("block %llu already unmapped\n",
+#ifdef DEBUG
+				printf("block %llu already unmapped\n",
 					logical);
+#endif
 				goto done;
 			}
 		} else
@@ -1154,19 +1191,25 @@ errcode_t ext2fs_extent_set_bmap(ext2_extent_handle_t handle,
 	/* check if already pointing to the requested physical */
 	if (mapped && (new_uninit == extent_uninit) &&
 	    (extent.e_pblk + (logical - extent.e_lblk) == physical)) {
-		dbg_printf("physical block (at %llu) unchanged\n", logical);
+#ifdef DEBUG
+		printf("physical block (at %llu) unchanged\n", logical);
+#endif
 		goto done;
 	}
 
 	if (!mapped) {
-		dbg_printf("mapping unmapped logical block %llu\n", logical);
+#ifdef DEBUG
+		printf("mapping unmapped logical block %llu\n", logical);
+#endif
 		if ((logical == extent.e_lblk + extent.e_len) &&
 		    (physical == extent.e_pblk + extent.e_len) &&
 		    (new_uninit == extent_uninit) &&
 		    ((int) extent.e_len < max_len-1)) {
 			extent.e_len++;
 			retval = ext2fs_extent_replace(handle, 0, &extent);
-		} else
+		} else if (logical < extent.e_lblk)
+			retval = ext2fs_extent_insert(handle, 0, &newextent);
+		else
 			retval = ext2fs_extent_insert(handle,
 				      EXT2_EXTENT_INSERT_AFTER, &newextent);
 		if (retval)
@@ -1175,10 +1218,11 @@ errcode_t ext2fs_extent_set_bmap(ext2_extent_handle_t handle,
 		if (retval)
 			goto done;
 	} else if ((logical == extent.e_lblk) && (extent.e_len == 1))  {
-		dbg_printf("(re/un)mapping only block in extent\n");
+#ifdef DEBUG
+		printf("(re/un)mapping only block in extent\n");
+#endif
 		if (physical) {
-			extent.e_pblk = physical;
-			retval = ext2fs_extent_replace(handle, 0, &extent);
+			retval = ext2fs_extent_replace(handle, 0, &newextent);
 		} else {
 			retval = ext2fs_extent_delete(handle, 0);
 			if (retval)
@@ -1191,7 +1235,9 @@ errcode_t ext2fs_extent_set_bmap(ext2_extent_handle_t handle,
 		if (retval)
 			goto done;
 	} else if (logical == extent.e_lblk + extent.e_len - 1)  {
-		dbg_printf("(re/un)mapping last block in extent\n");
+#ifdef DEBUG
+		printf("(re/un)mapping last block in extent\n");
+#endif
 		extent.e_len--;
 		retval = ext2fs_extent_replace(handle, 0, &extent);
 		if (retval)
@@ -1203,7 +1249,9 @@ errcode_t ext2fs_extent_set_bmap(ext2_extent_handle_t handle,
 				goto done;
 		}
 	} else if (logical == extent.e_lblk) {
-		dbg_printf("(re/un)mapping first block in extent\n");
+#ifdef DEBUG
+		printf("(re/un)mapping first block in extent\n");
+#endif
 		extent.e_pblk++;
 		extent.e_lblk++;
 		extent.e_len--;
@@ -1211,11 +1259,12 @@ errcode_t ext2fs_extent_set_bmap(ext2_extent_handle_t handle,
 		if (retval)
 			goto done;
 		if (physical) {
-			/* insert new extent ahead of current */
-			retval = ext2fs_extent_insert(handle,
-					0, &newextent);
-			if (retval)
-				goto done;
+			/*
+			 * We've removed the old block, now rely on
+			 * the optimized hueristics for adding a new
+			 * mapping with appropriate merging if necessary.
+			 */
+			goto again;
 		} else {
 			retval = ext2fs_extent_fix_parents(handle);
 			if (retval)
@@ -1224,7 +1273,9 @@ errcode_t ext2fs_extent_set_bmap(ext2_extent_handle_t handle,
 	} else {
 		__u32	orig_length;
 
-		dbg_printf("(re/un)mapping in middle of extent\n");
+#ifdef DEBUG
+		printf("(re/un)mapping in middle of extent\n");
+#endif
 		/* need to split this extent; later */
 
 		orig_length = extent.e_len;
@@ -1375,7 +1426,7 @@ int common_extent_args_process(int argc, char *argv[], int min_argc,
 	if (common_args_process(argc, argv, min_argc, max_argc, cmd,
 				usage, flags))
 		return 1;
-	
+
 	if (!current_handle) {
 		com_err(cmd, 0, "Extent handle not open");
 		return 1;
