@@ -4,8 +4,8 @@
  * Copyright (C) 1993, 1994, 1994, 1996 Theodore Ts'o.
  *
  * %Begin-Header%
- * This file may be redistributed under the terms of the GNU Library
- * General Public License, version 2.
+ * This file may be redistributed under the terms of the GNU Public
+ * License.
  * %End-Header%
  */
 
@@ -52,8 +52,7 @@ static errcode_t write_bitmaps(ext2_filsys fs, int do_inode, int do_block)
 	inode_nbytes = block_nbytes = 0;
 	if (do_block) {
 		block_nbytes = EXT2_BLOCKS_PER_GROUP(fs->super) / 8;
-		retval = ext2fs_get_memalign(fs->blocksize, fs->blocksize,
-					     &block_buf);
+		retval = ext2fs_get_mem(fs->blocksize, &block_buf);
 		if (retval)
 			return retval;
 		memset(block_buf, 0xff, fs->blocksize);
@@ -61,8 +60,7 @@ static errcode_t write_bitmaps(ext2_filsys fs, int do_inode, int do_block)
 	if (do_inode) {
 		inode_nbytes = (size_t)
 			((EXT2_INODES_PER_GROUP(fs->super)+7) / 8);
-		retval = ext2fs_get_memalign(fs->blocksize, fs->blocksize,
-					     &inode_buf);
+		retval = ext2fs_get_mem(fs->blocksize, &inode_buf);
 		if (retval)
 			return retval;
 		memset(inode_buf, 0xff, fs->blocksize);
@@ -171,13 +169,8 @@ static errcode_t read_bitmaps(ext2_filsys fs, int do_inode, int do_block)
 		retval = ext2fs_allocate_block_bitmap(fs, buf, &fs->block_map);
 		if (retval)
 			goto cleanup;
-		if (do_image)
-			retval = ext2fs_get_mem(fs->blocksize, &block_bitmap);
-		else
-			retval = ext2fs_get_memalign((unsigned) block_nbytes,
-						     fs->blocksize,
-						     &block_bitmap);
-			
+		retval = ext2fs_get_mem(do_image ? fs->blocksize :
+					(unsigned) block_nbytes, &block_bitmap);
 		if (retval)
 			goto cleanup;
 	} else
@@ -255,7 +248,7 @@ static errcode_t read_bitmaps(ext2_filsys fs, int do_inode, int do_block)
 					goto cleanup;
 				}
 			} else
-				memset(block_bitmap, 0, block_nbytes);
+				memset(block_bitmap, 0xff, block_nbytes);
 			cnt = block_nbytes << 3;
 			retval = ext2fs_set_block_bitmap_range(fs->block_map,
 					       blk_itr, cnt, block_bitmap);
@@ -277,7 +270,7 @@ static errcode_t read_bitmaps(ext2_filsys fs, int do_inode, int do_block)
 					goto cleanup;
 				}
 			} else
-				memset(inode_bitmap, 0, inode_nbytes);
+				memset(inode_bitmap, 0xff, inode_nbytes);
 			cnt = inode_nbytes << 3;
 			retval = ext2fs_set_inode_bitmap_range(fs->inode_map,
 					       ino_itr, cnt, inode_bitmap);
