@@ -4,8 +4,8 @@
  * Copyright (C) 1996 by Theodore Ts'o.
  *
  * %Begin-Header%
- * This file may be redistributed under the terms of the GNU Library
- * General Public License, version 2.
+ * This file may be redistributed under the terms of the GNU Public
+ * License.
  * %End-Header%
  */
 
@@ -271,8 +271,6 @@ int main(int argc, char **argv)
 	int	equal;
 	errcode_t	retval;
 
-	add_error_table(&et_ext2_error_table);
-
 	bb1 = bb2 = bb3 = bb4 = bb5 = 0;
 
 	printf("test1: ");
