@@ -20,7 +20,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <strings.h>
 #include <fcntl.h>
 #include <ctype.h>
 #include <time.h>
@@ -47,10 +46,8 @@ extern int optind;
 #endif
 #include <sys/ioctl.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <libgen.h>
 #include <limits.h>
-#include <blkid/blkid.h>
 
 #include "ext2fs/ext2_fs.h"
 #include "et/com_err.h"
@@ -80,7 +77,6 @@ int	cflag;
 int	verbose;
 int	quiet;
 int	super_only;
-int	discard = 1;	/* attempt to discard device before fs creation */
 int	force;
 int	noaction;
 int	journal_size;
@@ -90,7 +86,6 @@ char	*bad_blocks_filename;
 __u32	fs_stride;
 
 struct ext2_super_block fs_param;
-char *fs_uuid = NULL;
 char *creator_os;
 char *volume_label;
 char *mount_dir;
@@ -113,7 +108,7 @@ static void usage(void)
 	"\t[-g blocks-per-group] [-L volume-label] "
 	"[-M last-mounted-directory]\n\t[-O feature[,...]] "
 	"[-r fs-revision] [-E extended-option[,...]]\n"
-	"\t[-T fs-type] [-U UUID] [-jnqvFKSV] device [blocks-count]\n"),
+	"\t[-T fs-type] [-jnqvFSV] device [blocks-count]\n"),
 		program_name);
 	exit(1);
 }
@@ -350,7 +345,7 @@ static void progress_close(struct progress_struct *progress)
 	fputs(_("done                            \n"), stdout);
 }
 
-static void write_inode_tables(ext2_filsys fs, int lazy_flag, int itable_zeroed)
+static void write_inode_tables(ext2_filsys fs, int lazy_flag)
 {
 	errcode_t	retval;
 	blk_t		blk;
@@ -377,8 +372,7 @@ static void write_inode_tables(ext2_filsys fs, int lazy_flag, int itable_zeroed)
 				 EXT2_INODE_SIZE(fs->super)) +
 				EXT2_BLOCK_SIZE(fs->super) - 1) /
 			       EXT2_BLOCK_SIZE(fs->super));
-		}
-		if (!lazy_flag || itable_zeroed) {
+		} else {
 			/* The kernel doesn't need to zero the itable blocks */
 			fs->group_desc[i].bg_flags |= EXT2_BG_INODE_ZEROED;
 			ext2fs_group_desc_csum_set(fs, i);
@@ -618,8 +612,6 @@ static void show_stats(ext2_filsys fs)
 		s->s_log_block_size);
 	printf(_("Fragment size=%u (log=%u)\n"), fs->fragsize,
 		s->s_log_frag_size);
-	printf(_("Stride=%u blocks, Stripe width=%u blocks\n"),
-	       s->s_raid_stride, s->s_raid_stripe_width);
 	printf(_("%u inodes, %u blocks\n"), s->s_inodes_count,
 	       s->s_blocks_count);
 	printf(_("%u blocks (%2.2f%%) reserved for the super user\n"),
@@ -806,10 +798,6 @@ static void parse_extended_opts(struct ext2_super_block *param,
 				lazy_itable_init = strtoul(arg, &p, 0);
 			else
 				lazy_itable_init = 1;
-		} else if (!strcmp(token, "discard")) {
-			discard = 1;
-		} else if (!strcmp(token, "nodiscard")) {
-			discard = 0;
 		} else {
 			r_usage++;
 			badopt = token;
@@ -825,9 +813,7 @@ static void parse_extended_opts(struct ext2_super_block *param,
 			"\tstripe-width=<RAID stride * data disks in blocks>\n"
 			"\tresize=<resize maximum size in blocks>\n"
 			"\tlazy_itable_init=<0 to disable, 1 to enable>\n"
-			"\ttest_fs\n"
-			"\tdiscard\n"
-			"\tnodiscard\n\n"),
+			"\ttest_fs\n\n"),
 			badopt ? badopt : "");
 		free(buf);
 		exit(1);
@@ -934,35 +920,6 @@ static void print_str_list(char **list)
 	fputc('\n', stdout);
 }
 
-/*
- * Return TRUE if the profile has the given subsection
- */
-static int profile_has_subsection(profile_t profile, const char *section,
-				  const char *subsection)
-{
-	void			*state;
-	const char		*names[4];
-	char			*name;
-	int			ret = 0;
-
-	names[0] = section;
-	names[1] = subsection;
-	names[2] = 0;
-
-	if (profile_iterator_create(profile, names,
-				    PROFILE_ITER_LIST_SECTION |
-				    PROFILE_ITER_RELATIONS_ONLY, &state))
-		return 0;
-
-	if ((profile_iterator(&state, &name, 0) == 0) && name) {
-		free(name);
-		ret = 1;
-	}
-
-	profile_iterator_free(&state);
-	return ret;
-}
-
 static char **parse_fs_type(const char *fs_type,
 			    const char *usage_types,
 			    struct ext2_super_block *fs_param,
@@ -988,8 +945,6 @@ static char **parse_fs_type(const char *fs_type,
 		ext_type = fs_type;
 	else if (is_hurd)
 		ext_type = "ext2";
-	else if (!strcmp(program_name, "mke3fs"))
-		ext_type = "ext3";
 	else if (progname) {
 		ext_type = strrchr(progname, '/');
 		if (ext_type)
@@ -1013,19 +968,17 @@ static char **parse_fs_type(const char *fs_type,
 			ext_type = "ext3";
 	}
 
-
-	if (!profile_has_subsection(profile, "fs_types", ext_type) &&
-	    strcmp(ext_type, "ext2")) {
-		printf(_("\nYour mke2fs.conf file does not define the "
-			 "%s filesystem type.\n"), ext_type);
-		if (!strcmp(ext_type, "ext3") || !strcmp(ext_type, "ext4") ||
-		    !strcmp(ext_type, "ext4dev")) {
+	if (!strcmp(ext_type, "ext3") || !strcmp(ext_type, "ext4") ||
+	    !strcmp(ext_type, "ext4dev")) {
+		profile_get_string(profile, "fs_types", ext_type, "features",
+				   0, &t);
+		if (!t) {
+			printf(_("\nWarning!  Your mke2fs.conf file does "
+				 "not define the %s filesystem type.\n"),
+			       ext_type);
 			printf(_("You probably need to install an updated "
 				 "mke2fs.conf file.\n\n"));
-		}
-		if (!force) {
-			printf(_("Aborting...\n"));
-			exit(1);
+			sleep(5);
 		}
 	}
 
@@ -1058,15 +1011,8 @@ static char **parse_fs_type(const char *fs_type,
 		if (t)
 			*t = '\0';
 
-		if (*cp) {
-			if (profile_has_subsection(profile, "fs_types", cp))
-				push_string(&list, cp);
-			else if (strcmp(cp, "default") != 0)
-				fprintf(stderr,
-					_("\nWarning: the fs_type %s is not "
-					  "defined in mke2fs.conf\n\n"),
-					cp);
-		}
+		if (*cp)
+			push_string(&list, cp);
 		if (t)
 			cp = t+1;
 		else {
@@ -1075,7 +1021,8 @@ static char **parse_fs_type(const char *fs_type,
 		}
 	}
 	free(parse_str);
-	free(profile_type);
+	if (profile_type)
+		free(profile_type);
 	if (is_hurd)
 		push_string(&list, "hurd");
 	return (list.list);
@@ -1123,55 +1070,6 @@ static int get_bool_from_profile(char **fs_types, const char *opt, int def_val)
 extern const char *mke2fs_default_profile;
 static const char *default_files[] = { "<default>", 0 };
 
-#ifdef HAVE_BLKID_PROBE_GET_TOPOLOGY
-/*
- * Sets the geometry of a device (stripe/stride), and returns the
- * device's alignment offset, if any, or a negative error.
- */
-static int get_device_geometry(const char *file,
-			       struct ext2_super_block *fs_param,
-			       int psector_size)
-{
-	int rc = -1;
-	int blocksize;
-	blkid_probe pr;
-	blkid_topology tp;
-	unsigned long min_io;
-	unsigned long opt_io;
-	struct stat statbuf;
-
-	/* Nothing to do for a regular file */
-	if (!stat(file, &statbuf) && S_ISREG(statbuf.st_mode))
-		return 0;
-
-	pr = blkid_new_probe_from_filename(file);
-	if (!pr)
-		goto out;
-
-	tp = blkid_probe_get_topology(pr);
-	if (!tp)
-		goto out;
-
-	min_io = blkid_topology_get_minimum_io_size(tp);
-	opt_io = blkid_topology_get_optimal_io_size(tp);
-	blocksize = EXT2_BLOCK_SIZE(fs_param);
-	if ((min_io == 0) && (psector_size > blocksize))
-		min_io = psector_size;
-	if ((opt_io == 0) && min_io)
-		opt_io = min_io;
-	if ((opt_io == 0) && (psector_size > blocksize))
-		opt_io = psector_size;
-
-	fs_param->s_raid_stride = min_io / blocksize;
-	fs_param->s_raid_stripe_width = opt_io / blocksize;
-
-	rc = blkid_topology_get_alignment_offset(tp);
-out:
-	blkid_free_probe(pr);
-	return rc;
-}
-#endif
-
 static void PRS(int argc, char *argv[])
 {
 	int		b, c;
@@ -1182,7 +1080,7 @@ static void PRS(int argc, char *argv[])
 	int		inode_size = 0;
 	unsigned long	flex_bg_size = 0;
 	double		reserved_ratio = 5.0;
-	int		lsector_size = 0, psector_size = 0;
+	int		sector_size = 0;
 	int		show_version_only = 0;
 	unsigned long long num_inodes = 0; /* unsigned long long to catch too-large input */
 	errcode_t	retval;
@@ -1259,13 +1157,12 @@ static void PRS(int argc, char *argv[])
 		program_name = get_progname(*argv);
 
 		/* If called as mkfs.ext3, create a journal inode */
-		if (!strcmp(program_name, "mkfs.ext3") ||
-		    !strcmp(program_name, "mke3fs"))
+		if (!strcmp(program_name, "mkfs.ext3"))
 			journal_size = -1;
 	}
 
 	while ((c = getopt (argc, argv,
-		    "b:cf:g:G:i:jl:m:no:qr:s:t:vE:FI:J:KL:M:N:O:R:ST:U:V")) != EOF) {
+		    "b:cf:g:G:i:jl:m:no:qr:s:t:vE:FI:J:L:M:N:O:R:ST:V")) != EOF) {
 		switch (c) {
 		case 'b':
 			blocksize = strtol(optarg, &tmp, 0);
@@ -1322,7 +1219,7 @@ static void PRS(int argc, char *argv[])
 					_("Illegal number for flex_bg size"));
 				exit(1);
 			}
-			if (flex_bg_size < 1 ||
+			if (flex_bg_size < 2 ||
 			    (flex_bg_size & (flex_bg_size-1)) != 0) {
 				com_err(program_name, 0,
 					_("flex_bg size must be a power of 2"));
@@ -1337,20 +1234,13 @@ static void PRS(int argc, char *argv[])
 				com_err(program_name, 0,
 					_("invalid inode ratio %s (min %d/max %d)"),
 					optarg, EXT2_MIN_BLOCK_SIZE,
-					EXT2_MAX_BLOCK_SIZE * 1024);
+					EXT2_MAX_BLOCK_SIZE);
 				exit(1);
 			}
 			break;
 		case 'J':
 			parse_journal_opts(optarg);
 			break;
-		case 'K':
-			fprintf(stderr, _("Warning: -K option is deprecated and "
-					  "should not be used anymore. Use "
-					  "\'-E nodiscard\' extended option "
-					  "instead!\n"));
-			discard = 0;
-			break;
 		case 'j':
 			if (!journal_size)
 				journal_size = -1;
@@ -1366,8 +1256,7 @@ static void PRS(int argc, char *argv[])
 			break;
 		case 'm':
 			reserved_ratio = strtod(optarg, &tmp);
-			if ( *tmp || reserved_ratio > 50 ||
-			     reserved_ratio < 0) {
+			if (reserved_ratio > 50 || *tmp) {
 				com_err(program_name, 0,
 					_("invalid reserved blocks percent - %s"),
 					optarg);
@@ -1439,9 +1328,6 @@ static void PRS(int argc, char *argv[])
 		case 'T':
 			usage_types = optarg;
 			break;
-		case 'U':
-			fs_uuid = optarg;
-			break;
 		case 'V':
 			/* Print version number and exit */
 			show_version_only++;
@@ -1517,9 +1403,8 @@ static void PRS(int argc, char *argv[])
 		fs_param.s_blocks_count = parse_num_blocks(argv[optind++],
 				fs_param.s_log_block_size);
 		if (!fs_param.s_blocks_count) {
-			com_err(program_name, 0,
-				_("invalid blocks count '%s' on device '%s'"),
-				argv[optind - 1], device_name);
+			com_err(program_name, 0, _("invalid blocks count - %s"),
+				argv[optind - 1]);
 			exit(1);
 		}
 	}
@@ -1550,33 +1435,17 @@ static void PRS(int argc, char *argv[])
 	}
 
 	if (retval == EFBIG) {
-		blk64_t	big_dev_size;
-
-		if (blocksize < 4096) {
-			fs_param.s_log_block_size = 2;
-			blocksize = 4096;
-		}
-		retval = ext2fs_get_device_size2(device_name,
-				 EXT2_BLOCK_SIZE(&fs_param), &big_dev_size);
-		if (retval)
-			goto get_size_failure;
-		if (big_dev_size == (1ULL << 32)) {
-			dev_size = (blk_t) (big_dev_size - 1);
-			goto got_size;
-		}
 		fprintf(stderr, _("%s: Size of device %s too big "
 				  "to be expressed in 32 bits\n\t"
 				  "using a blocksize of %d.\n"),
 			program_name, device_name, EXT2_BLOCK_SIZE(&fs_param));
 		exit(1);
 	}
-get_size_failure:
 	if (retval && (retval != EXT2_ET_UNIMPLEMENTED)) {
 		com_err(program_name, retval,
 			_("while trying to determine filesystem size"));
 		exit(1);
 	}
-got_size:
 	if (!fs_param.s_blocks_count) {
 		if (retval == EXT2_ET_UNIMPLEMENTED) {
 			com_err(program_name, 0,
@@ -1630,14 +1499,16 @@ got_size:
 					   "features", "", &tmp);
 			if (tmp && *tmp)
 				edit_feature(tmp, &fs_param.s_feature_compat);
-			free(tmp);
+			if (tmp)
+				free(tmp);
 		}
 		tmp = get_string_from_profile(fs_types, "default_features",
 					      "");
 	}
 	edit_feature(fs_features ? fs_features : tmp,
 		     &fs_param.s_feature_compat);
-	free(tmp);
+	if (tmp)
+		free(tmp);
 
 	if (fs_param.s_feature_incompat & EXT3_FEATURE_INCOMPAT_JOURNAL_DEV) {
 		fs_types[0] = strdup("journal");
@@ -1702,28 +1573,16 @@ got_size:
 	    ((tmp = getenv("MKE2FS_FIRST_META_BG"))))
 		fs_param.s_first_meta_bg = atoi(tmp);
 
-	/* Get the hardware sector sizes, if available */
-	retval = ext2fs_get_device_sectsize(device_name, &lsector_size);
+	/* Get the hardware sector size, if available */
+	retval = ext2fs_get_device_sectsize(device_name, &sector_size);
 	if (retval) {
 		com_err(program_name, retval,
 			_("while trying to determine hardware sector size"));
 		exit(1);
 	}
-	retval = ext2fs_get_device_phys_sectsize(device_name, &psector_size);
-	if (retval) {
-		com_err(program_name, retval,
-			_("while trying to determine physical sector size"));
-		exit(1);
-	}
 
 	if ((tmp = getenv("MKE2FS_DEVICE_SECTSIZE")) != NULL)
-		lsector_size = atoi(tmp);
-	if ((tmp = getenv("MKE2FS_DEVICE_PHYS_SECTSIZE")) != NULL)
-		psector_size = atoi(tmp);
-
-	/* Older kernels may not have physical/logical distinction */
-	if (!psector_size)
-		psector_size = lsector_size;
+		sector_size = atoi(tmp);
 
 	if (blocksize <= 0) {
 		use_bsize = get_int_from_profile(fs_types, "blocksize", 4096);
@@ -1734,24 +1593,12 @@ got_size:
 			    (use_bsize > 4096))
 				use_bsize = 4096;
 		}
-		if (lsector_size && use_bsize < lsector_size)
-			use_bsize = lsector_size;
+		if (sector_size && use_bsize < sector_size)
+			use_bsize = sector_size;
 		if ((blocksize < 0) && (use_bsize < (-blocksize)))
 			use_bsize = -blocksize;
 		blocksize = use_bsize;
 		fs_param.s_blocks_count /= blocksize / 1024;
-	} else {
-		if (blocksize < lsector_size) {			/* Impossible */
-			com_err(program_name, EINVAL,
-				_("while setting blocksize; too small "
-				  "for device\n"));
-			exit(1);
-		} else if ((blocksize < psector_size) &&
-			   (psector_size <= sys_page_size)) {	/* Suboptimal */
-			fprintf(stderr, _("Warning: specified blocksize %d is "
-				"less than device physical sectorsize %d\n"),
-				blocksize, psector_size);
-		}
 	}
 
 	if (inode_ratio == 0) {
@@ -1764,30 +1611,10 @@ got_size:
 	fs_param.s_log_frag_size = fs_param.s_log_block_size =
 		int_log2(blocksize >> EXT2_MIN_BLOCK_LOG_SIZE);
 
-#ifdef HAVE_BLKID_PROBE_GET_TOPOLOGY
-	retval = get_device_geometry(device_name, &fs_param, psector_size);
-	if (retval < 0) {
-		fprintf(stderr,
-			_("warning: Unable to get device geometry for %s\n"),
-			device_name);
-	} else if (retval) {
-		printf(_("%s alignment is offset by %lu bytes.\n"),
-		       device_name, retval);
-		printf(_("This may result in very poor performance, "
-			  "(re)-partitioning suggested.\n"));
-	}
-#endif
-
 	blocksize = EXT2_BLOCK_SIZE(&fs_param);
 
-	lazy_itable_init = 0;
-	if (access("/sys/fs/ext4/features/lazy_itable_init", R_OK) == 0)
-		lazy_itable_init = 1;
-
 	lazy_itable_init = get_bool_from_profile(fs_types,
-						 "lazy_itable_init",
-						 lazy_itable_init);
-	discard = get_bool_from_profile(fs_types, "discard" , discard);
+						 "lazy_itable_init", 0);
 
 	/* Get options from profile */
 	for (cpp = fs_types; *cpp; cpp++) {
@@ -1795,7 +1622,8 @@ got_size:
 		profile_get_string(profile, "fs_types", *cpp, "options", "", &tmp);
 			if (tmp && *tmp)
 				parse_extended_opts(&fs_param, tmp);
-			free(tmp);
+			if (tmp)
+				free(tmp);
 	}
 
 	if (extended_opts)
@@ -1949,7 +1777,7 @@ open_err_out:
 static int mke2fs_setup_tdb(const char *name, io_manager *io_ptr)
 {
 	errcode_t retval = 0;
-	char *tdb_dir, *tdb_file;
+	char *tdb_dir, tdb_file[PATH_MAX];
 	char *device_name, *tmp_name;
 
 	/*
@@ -1967,16 +1795,7 @@ static int mke2fs_setup_tdb(const char *name, io_manager *io_ptr)
 		return 0;
 
 	tmp_name = strdup(name);
-	if (!tmp_name) {
-	alloc_fn_fail:
-		com_err(program_name, ENOMEM, 
-			_("Couldn't allocate memory for tdb filename\n"));
-		return ENOMEM;
-	}
 	device_name = basename(tmp_name);
-	tdb_file = malloc(strlen(tdb_dir) + 8 + strlen(device_name) + 7 + 1);
-	if (!tdb_file)
-		goto alloc_fn_fail;
 	sprintf(tdb_file, "%s/mke2fs-%s.e2undo", tdb_dir, device_name);
 
 	if (!access(tdb_file, F_OK)) {
@@ -1985,7 +1804,6 @@ static int mke2fs_setup_tdb(const char *name, io_manager *io_ptr)
 			com_err(program_name, retval,
 				_("while trying to delete %s"),
 				tdb_file);
-			free(tdb_file);
 			return retval;
 		}
 	}
@@ -1997,74 +1815,10 @@ static int mke2fs_setup_tdb(const char *name, io_manager *io_ptr)
 		 "using the command:\n"
 		 "    e2undo %s %s\n\n"), tdb_file, name);
 
-	free(tdb_file);
 	free(tmp_name);
 	return retval;
 }
 
-#ifdef __linux__
-
-#ifndef BLKDISCARD
-#define BLKDISCARD	_IO(0x12,119)
-#endif
-
-#ifndef BLKDISCARDZEROES
-#define BLKDISCARDZEROES _IO(0x12,124)
-#endif
-
-/*
- * Return zero if the discard succeeds, and -1 if the discard fails.
- */
-static int mke2fs_discard_blocks(ext2_filsys fs)
-{
-	int fd;
-	int ret;
-	int blocksize;
-	__u64 blocks;
-	__uint64_t range[2];
-
-	blocks = fs->super->s_blocks_count;
-	blocksize = EXT2_BLOCK_SIZE(fs->super);
-	range[0] = 0;
-	range[1] = blocks * blocksize;
-
-	fd = open64(fs->device_name, O_RDWR);
-
-	if (fd > 0) {
-		ret = ioctl(fd, BLKDISCARD, &range);
-		if (verbose) {
-			printf(_("Calling BLKDISCARD from %llu to %llu "),
-			       (unsigned long long) range[0],
-			       (unsigned long long) range[1]);
-			if (ret)
-				printf(_("failed.\n"));
-			else
-				printf(_("succeeded.\n"));
-		}
-		close(fd);
-	}
-	return ret;
-}
-
-static int mke2fs_discard_zeroes_data(ext2_filsys fs)
-{
-	int fd;
-	int ret;
-	int discard_zeroes_data = 0;
-
-	fd = open64(fs->device_name, O_RDWR);
-
-	if (fd > 0) {
-		ioctl(fd, BLKDISCARDZEROES, &discard_zeroes_data);
-		close(fd);
-	}
-	return discard_zeroes_data;
-}
-#else
-#define mke2fs_discard_blocks(fs)	1
-#define mke2fs_discard_zeroes_data(fs)	0
-#endif
-
 int main (int argc, char *argv[])
 {
 	errcode_t	retval = 0;
@@ -2076,7 +1830,6 @@ int main (int argc, char *argv[])
 	io_manager	io_ptr;
 	char		tdb_string[40];
 	char		*hash_alg_str;
-	int		itable_zeroed = 0;
 
 #ifdef ENABLE_NLS
 	setlocale(LC_MESSAGES, "");
@@ -2109,20 +1862,6 @@ int main (int argc, char *argv[])
 		com_err(device_name, retval, _("while setting up superblock"));
 		exit(1);
 	}
-
-	/* Can't undo discard ... */
-	if (discard && (io_ptr != undo_io_manager)) {
-		retval = mke2fs_discard_blocks(fs);
-
-		if (!retval && mke2fs_discard_zeroes_data(fs)) {
-			if (verbose)
-				printf(_("Discard succeeded and will return 0s "
-					 " - skipping inode table wipe\n"));
-			lazy_itable_init = 1;
-			itable_zeroed = 1;
-		}
-	}
-
 	sprintf(tdb_string, "tdb_data_size=%d", fs->blocksize <= 4096 ?
 		32768 : fs->blocksize * 8);
 	io_channel_set_options(fs->io, tdb_string);
@@ -2130,14 +1869,6 @@ int main (int argc, char *argv[])
 	if (fs_param.s_flags & EXT2_FLAGS_TEST_FILESYS)
 		fs->super->s_flags |= EXT2_FLAGS_TEST_FILESYS;
 
-	if ((fs_param.s_feature_incompat &
-	     (EXT3_FEATURE_INCOMPAT_EXTENTS|EXT4_FEATURE_INCOMPAT_FLEX_BG)) ||
-	    (fs_param.s_feature_ro_compat &
-	     (EXT4_FEATURE_RO_COMPAT_HUGE_FILE|EXT4_FEATURE_RO_COMPAT_GDT_CSUM|
-	      EXT4_FEATURE_RO_COMPAT_DIR_NLINK|
-	      EXT4_FEATURE_RO_COMPAT_EXTRA_ISIZE)))
-		fs->super->s_kbytes_written = 1;
-
 	/*
 	 * Wipe out the old on-disk superblock
 	 */
@@ -2145,16 +1876,9 @@ int main (int argc, char *argv[])
 		zap_sector(fs, 2, 6);
 
 	/*
-	 * Parse or generate a UUID for the filesystem
+	 * Generate a UUID for it...
 	 */
-	if (fs_uuid) {
-		if (uuid_parse(fs_uuid, fs->super->s_uuid) !=0) {
-			com_err(device_name, 0, "could not parse UUID: %s\n",
-				fs_uuid);
-			exit(1);
-		}
-	} else
-		uuid_generate(fs->super->s_uuid);
+	uuid_generate(fs->super->s_uuid);
 
 	/*
 	 * Initialize the directory index variables
@@ -2162,7 +1886,6 @@ int main (int argc, char *argv[])
 	hash_alg_str = get_string_from_profile(fs_types, "hash_alg",
 					       "half_md4");
 	hash_alg = e2p_string2hash(hash_alg_str);
-	free(hash_alg_str);
 	fs->super->s_def_hash_version = (hash_alg >= 0) ? hash_alg :
 		EXT2_HASH_HALF_MD4;
 	uuid_generate((unsigned char *) fs->super->s_hash_seed);
@@ -2268,7 +1991,7 @@ int main (int argc, char *argv[])
 				_("while zeroing block %u at end of filesystem"),
 				ret_blk);
 		}
-		write_inode_tables(fs, lazy_itable_init, itable_zeroed);
+		write_inode_tables(fs, lazy_itable_init);
 		create_root_dir(fs);
 		create_lost_and_found(fs);
 		reserve_inodes(fs);
@@ -2321,12 +2044,6 @@ int main (int argc, char *argv[])
 		    EXT3_FEATURE_COMPAT_HAS_JOURNAL)) {
 		journal_blocks = figure_journal_size(journal_size, fs);
 
-		if (super_only) {
-			printf(_("Skipping journal creation in super-only mode\n"));
-			fs->super->s_journal_inum = EXT2_JOURNAL_INO;
-			goto no_journal;
-		}
-
 		if (!journal_blocks) {
 			fs->super->s_feature_compat &=
 				~EXT3_FEATURE_COMPAT_HAS_JOURNAL;
@@ -2366,8 +2083,5 @@ no_journal:
 	remove_error_table(&et_ext2_error_table);
 	remove_error_table(&et_prof_error_table);
 	profile_release(profile);
-	for (i=0; fs_types[i]; i++)
-		free(fs_types[i]);
-	free(fs_types);
 	return (retval || val) ? 1 : 0;
 }
