@@ -54,8 +54,10 @@ static int add_subst(char *name, char *value)
 	return 0;
 fail:
 	if (ent) {
-		free(ent->name);
-		free(ent->value);
+		if (ent->name)
+			free(ent->name);
+		if (ent->value)
+			free(ent->value);
 		free(ent);
 	}
 	return retval;
